@@ -19,9 +19,9 @@ package MongoDB::ClientSession;
 # ABSTRACT: MongoDB session and transaction management
 
 use version;
-our $VERSION = 'v2.1.0'; # TRIAL
+our $VERSION = 'v2.1.1'; # TRIAL
 
-use MongoDB::Error;
+use MongoDB::Error 'EXCEEDED_TIME_LIMIT';
 
 use Moo;
 use MongoDB::_Constants;
@@ -30,14 +30,19 @@ use MongoDB::_Types qw(
     BSONTimestamp
     TransactionState
     Boolish
+    HostAddress
 );
 use Types::Standard qw(
     Maybe
     HashRef
     InstanceOf
+    Int
 );
 use MongoDB::_TransactionOptions;
+use Time::HiRes ();
 use namespace::clean -except => 'meta';
+use MongoDB::Op::_EndTxn;
+use Safe::Isa;
 
 #pod =attr client
 #pod
@@ -116,12 +121,19 @@ has _current_transaction_options => (
     is => 'rwp',
     isa => InstanceOf[ 'MongoDB::_TransactionOptions' ],
     handles  => {
-        _get_transaction_write_concern   => 'write_concern',
-        _get_transaction_read_concern    => 'read_concern',
-        _get_transaction_read_preference => 'read_preference',
+        _get_transaction_write_concern      => 'write_concern',
+        _get_transaction_read_concern       => 'read_concern',
+        _get_transaction_read_preference    => 'read_preference',
+        _get_transaction_max_commit_time_ms => 'max_commit_time_ms',
     },
 );
 
+has _address => (
+    is  => 'rwp',
+    isa => HostAddress,
+    clearer => '_unpin_address',
+);
+
 has _transaction_state => (
     is => 'rwp',
     isa => TransactionState,
@@ -143,6 +155,14 @@ has _has_transaction_operations => (
     default => 0,
 );
 
+# Used for retries of commit transactions - also set during abort transaction
+# but that cant be retried
+has _has_attempted_end_transaction => (
+    is       => 'rw',
+    isa      => Boolish,
+    default  => 0,
+);
+
 #pod =attr operation_time
 #pod
 #pod The last operation time. This is updated when an operation is performed during
@@ -158,6 +178,14 @@ has operation_time => (
     default => undef,
 );
 
+# Used in recovery of transactions on a sharded cluster
+has _recovery_token => (
+    is       => 'rwp',
+    isa      => Maybe[Document],
+    init_arg => undef,
+    default  => undef,
+);
+
 #pod =method session_id
 #pod
 #pod The session id for this particular session.  This should be considered
@@ -296,6 +324,9 @@ sub _in_transaction_state {
 #pod   C<defaultTransactionOptions> or from the parent client. This value will
 #pod   override all other read preferences set in any subsequent commands inside this
 #pod   transaction.
+#pod * C<maxCommitTimeMS> - The C<maxCommitTimeMS> specifies a cumulative time limit in
+#pod   milliseconds for processing operations on the cursor. MongoDB interrupts the
+#pod   operation at the earliest following interrupt point.
 #pod
 #pod =cut
 
@@ -321,8 +352,10 @@ sub start_transaction {
 
     $self->_increment_transaction_id;
 
+    $self->_unpin_address;
     $self->_set__active_transaction( 1 );
     $self->_set__has_transaction_operations( 0 );
+    $self->_has_attempted_end_transaction( 0 );
 
     return;
 }
@@ -380,8 +413,12 @@ sub commit_transaction {
     # until after another command has been called using this session
     $self->_set__active_transaction( 1 );
 
+    my $max_time_ms = $self->_get_transaction_max_commit_time_ms;
     eval {
-        $self->_send_end_transaction_command( TXN_COMMITTED, [ commitTransaction => 1 ] );
+        $self->_send_end_transaction_command( TXN_COMMITTED, [
+            commitTransaction => 1,
+            defined($max_time_ms) ? (maxTimeMS => $max_time_ms) : ()
+        ] );
     };
     if ( my $err = $@ ) {
         # catch and re-throw after retryable errors
@@ -414,7 +451,8 @@ sub commit_transaction {
                 251, # NoSuchTransaction
             )
         ) {
-            push @{ $err->error_labels }, 'UnknownTransactionCommitResult';
+            push @{ $err->error_labels }, TXN_UNKNOWN_COMMIT_MSG
+                unless $err->has_error_label( TXN_UNKNOWN_COMMIT_MSG );
         }
         die $err;
     }
@@ -446,10 +484,13 @@ sub abort_transaction {
     MongoDB::UsageError->throw("Cannot call abort_transaction twice")
         if $self->_in_transaction_state( TXN_ABORTED );
 
+    # Ignore all errors thrown by abortTransaction
     eval {
         $self->_send_end_transaction_command( TXN_ABORTED, [ abortTransaction => 1 ] );
     };
-    # Ignore all errors thrown by abortTransaction
+
+    # Make sure active transaction is turned off, even when the command itself fails
+    $self->_set__active_transaction( 0 );
 
     return;
 }
@@ -461,17 +502,15 @@ sub _send_end_transaction_command {
 
     # Only need to send commit command if the transaction actually sent anything
     if ( $self->_has_transaction_operations ) {
-        my $op = MongoDB::Op::_Command->_new(
+        my $op = MongoDB::Op::_EndTxn->_new(
             db_name             => 'admin',
             query               => $command,
-            query_flags         => {},
             bson_codec          => $self->client->bson_codec,
             session             => $self,
             monitoring_callback => $self->client->monitoring_callback,
         );
 
         my $result = $self->client->send_retryable_write_op( $op, 'force' );
-        $result->assert_no_write_concern_error;
     }
 
     # If the commit/abort succeeded, we are no longer in an active transaction
@@ -479,15 +518,36 @@ sub _send_end_transaction_command {
 }
 
 # For applying connection errors etc
-sub _maybe_apply_error_labels {
+sub _maybe_apply_error_labels_and_unpin {
     my ( $self, $err ) = @_;
 
     if ( $self->_in_transaction_state( TXN_STARTING, TXN_IN_PROGRESS ) ) {
-        push @{ $err->error_labels }, 'TransientTransactionError';
+        $err->add_error_label( TXN_TRANSIENT_ERROR_MSG )
+            if $err->$_isa("MongoDB::Error") && $err->_is_transient_transaction_error;
+    } elsif ( $self->_in_transaction_state( TXN_COMMITTED ) ) {
+        $err->add_error_label( TXN_UNKNOWN_COMMIT_MSG )
+            if $err->$_isa("MongoDB::Error") && $err->_is_unknown_commit_error;
     }
+    $self->_maybe_unpin_address( $err->error_labels );
     return;
 }
 
+# Passed an arrayref of error labels. Used where the client session isnt actively
+# adding the label (like from the database, in CommandResult), nor is the
+# calling class able to pass a constructed error
+sub _maybe_unpin_address {
+    my ( $self, $error_labels ) = @_;
+
+    my %labels = ( map { $_ => 1 } @$error_labels );
+    if ( $labels{ +TXN_TRANSIENT_ERROR_MSG }
+      # Must also unpin if its an unknown commit error during a commit
+      || ( $self->_in_transaction_state( TXN_COMMITTED )
+        && $labels{ +TXN_UNKNOWN_COMMIT_MSG } )
+    ) {
+        $self->_unpin_address;
+    }
+}
+
 #pod =method end_session
 #pod
 #pod     $session->end_session;
@@ -513,6 +573,140 @@ sub end_session {
     }
 }
 
+#pod =method with_transaction
+#pod
+#pod     $session->with_transaction($callback, $options);
+#pod
+#pod Execute a callback in a transaction.
+#pod
+#pod This method starts a transaction on this session, executes C<$callback>, and
+#pod then commits the transaction, returning the return value of the C<$callback>.
+#pod The C<$callback> will be executed at least once.
+#pod
+#pod If the C<$callback> throws an error, the transaction will be aborted. If less
+#pod than 120 seconds have passed since calling C<with_transaction>, and the error
+#pod has a C<TransientTransactionError> label, the transaction will be restarted and
+#pod the callback will be executed again. Otherwise, the error will be thrown.
+#pod
+#pod If the C<$callback> succeeds, then the transaction will be committed. If an
+#pod error is thrown from committing the transaction, and it is less than 120
+#pod seconds since calling C<with_transaction>, then:
+#pod
+#pod =for :list
+#pod * If the error has a C<TransientTransactionError> label, the transaction will be
+#pod   restarted.
+#pod * If the error has an C<UnknownTransactionCommitResult> label, and is not a
+#pod   C<MaxTimeMSExpired> error, then the commit will be retried.
+#pod
+#pod If the C<$callback> aborts or commits the transaction, no other actions are
+#pod taken and the return value of the C<$callback> is returned.
+#pod
+#pod The callback is called with the first (and only) argument being the session,
+#pod after starting the transaction:
+#pod
+#pod     $session->with_transaction( sub {
+#pod         # this is the same session as used for with_transaction
+#pod         my $cb_session = shift;
+#pod         ...
+#pod     }, $options);
+#pod
+#pod To pass arbitrary arguments to the C<$callback>, wrap your callback in a coderef:
+#pod
+#pod     $session->with_transaction(sub { $callback->($session, $foo, ...) }, $options);
+#pod
+#pod B<Warning>: you must either use the provided session within the callback, or
+#pod otherwise pass the session in use to the callback. You must pass the
+#pod C<$session> as an option to all database operations that need to be included
+#pod in the transaction.
+#pod
+#pod B<Warning>: The C<$callback> can be called multiple times, so it is recommended
+#pod to make it idempotent.
+#pod
+#pod A hash reference of options may be provided. these are the same as for
+#pod L</start_transaction>.
+#pod
+#pod =cut
+
+# We may not have a monotonic clock, but must use one for checking time limits
+my $HAS_MONOTONIC = eval { Time::HiRes::clock_gettime(Time::HiRes::CLOCK_MONOTONIC()); 1 };
+*monotonic_time = $HAS_MONOTONIC ? sub { Time::HiRes::clock_gettime(Time::HiRes::CLOCK_MONOTONIC()) } : \&Time::HiRes::time;
+
+sub _within_time_limit {
+    my ($self, $start_time) = @_;
+    return monotonic_time() - $start_time < WITH_TXN_RETRY_TIME_LIMIT;
+}
+
+sub _is_commit_timeout_error {
+    my ($self, $err) = @_;
+    if ( $err->can('result') && $err->result->can('output') ) {
+        my $output = $err->result->output;
+        my $err_code = $output->{ code };
+        my $err_codename = $output->{ codeName };
+        if ( defined $output->{ writeConcernError } ) {
+            $err_code = $output->{ writeConcernError }->{ code };
+            $err_codename = $output->{ writeConcernError }->{ codeName };
+        }
+        return 1 if ( $err_code == EXCEEDED_TIME_LIMIT ) || ( $err_codename eq 'MaxTimeMSExpired' );
+    }
+    return;
+}
+
+sub with_transaction {
+    my ( $self, $callback, $options ) = @_;
+    my $start_time = monotonic_time();
+    TRANSACTION: while (1) {
+        $self->start_transaction($options);
+
+        my $ret = eval { $callback->($self) };
+        if (my $err = $@) {
+            if ( $self->_in_transaction_state(TXN_STARTING, TXN_IN_PROGRESS) ) {
+                # Ignore all errors
+                eval { $self->abort_transaction };
+            }
+            if ( $err->$_isa('MongoDB::Error')
+              && $err->has_error_label(TXN_TRANSIENT_ERROR_MSG)
+              && $self->_within_time_limit($start_time) ) {
+                # Set inactive transaction to force transaction id to increment on next start
+                $self->_set__active_transaction(0);
+                next TRANSACTION;
+            }
+            die $err;
+        }
+        if ( $self->_in_transaction_state(TXN_NONE, TXN_COMMITTED, TXN_ABORTED) ) {
+            # Assume callback intentionally ended the transaction
+            return $ret;
+        }
+
+        COMMIT: while (1) {
+            eval { $self->commit_transaction };
+            if (my $err = $@) {
+                if ( $err->$_isa('MongoDB::Error') ) {
+                    if ( $self->_within_time_limit($start_time) ) {
+                        # Order is important here - a transient transaction
+                        # error means the entire transaction may have gone
+                        # wrong, whereas an unknown commit means only the
+                        # commit may have failed.
+                        if ( $err->has_error_label(TXN_TRANSIENT_ERROR_MSG) ) {
+                            # Set inactive transaction to force transaction id to increment on next start
+                            $self->_set__active_transaction(0);
+                            next TRANSACTION;
+                        }
+                        if ( $err->has_error_label(TXN_UNKNOWN_COMMIT_MSG)
+                             && ! $self->_is_commit_timeout_error( $err ) )
+                        {
+                            next COMMIT;
+                        }
+
+                    }
+                }
+                die $err;
+            }
+            # Commit succeeded
+            return $ret;
+        }
+    }
+}
+
 sub DEMOLISH {
     my ( $self, $in_global_destruction ) = @_;
     # Implicit end of session in scope
@@ -533,7 +727,7 @@ MongoDB::ClientSession - MongoDB session and transaction management
 
 =head1 VERSION
 
-version v2.1.0
+version v2.1.1
 
 =head1 SYNOPSIS
 
@@ -578,6 +772,10 @@ cursor will raise an error.
 
 =head2 Thread Safety
 
+B<NOTE>: Per L<threads> documentation, use of Perl threads is discouraged by the
+maintainers of Perl and the MongoDB Perl driver does not test or provide support
+for use with threads.
+
 Sessions are NOT thread safe, and should only be used by one thread at a time.
 Using a session across multiple threads is unsupported and unexpected issues
 and errors may occur. Note that the driver does not check for multi-threaded
@@ -689,6 +887,10 @@ C<writeConcern> - The write concern to use for committing or aborting this trans
 
 C<readPreference> - The read preference to use for all read operations in this transaction. If not defined, then will inherit from C<defaultTransactionOptions> or from the parent client. This value will override all other read preferences set in any subsequent commands inside this transaction.
 
+=item *
+
+C<maxCommitTimeMS> - The C<maxCommitTimeMS> specifies a cumulative time limit in milliseconds for processing operations on the cursor. MongoDB interrupts the operation at the earliest following interrupt point.
+
 =back
 
 =head2 commit_transaction
@@ -738,6 +940,64 @@ effect after calling for the first time.
 
 This will be called automatically by the object destructor.
 
+=head2 with_transaction
+
+    $session->with_transaction($callback, $options);
+
+Execute a callback in a transaction.
+
+This method starts a transaction on this session, executes C<$callback>, and
+then commits the transaction, returning the return value of the C<$callback>.
+The C<$callback> will be executed at least once.
+
+If the C<$callback> throws an error, the transaction will be aborted. If less
+than 120 seconds have passed since calling C<with_transaction>, and the error
+has a C<TransientTransactionError> label, the transaction will be restarted and
+the callback will be executed again. Otherwise, the error will be thrown.
+
+If the C<$callback> succeeds, then the transaction will be committed. If an
+error is thrown from committing the transaction, and it is less than 120
+seconds since calling C<with_transaction>, then:
+
+=over 4
+
+=item *
+
+If the error has a C<TransientTransactionError> label, the transaction will be restarted.
+
+=item *
+
+If the error has an C<UnknownTransactionCommitResult> label, and is not a C<MaxTimeMSExpired> error, then the commit will be retried.
+
+=back
+
+If the C<$callback> aborts or commits the transaction, no other actions are
+taken and the return value of the C<$callback> is returned.
+
+The callback is called with the first (and only) argument being the session,
+after starting the transaction:
+
+    $session->with_transaction( sub {
+        # this is the same session as used for with_transaction
+        my $cb_session = shift;
+        ...
+    }, $options);
+
+To pass arbitrary arguments to the C<$callback>, wrap your callback in a coderef:
+
+    $session->with_transaction(sub { $callback->($session, $foo, ...) }, $options);
+
+B<Warning>: you must either use the provided session within the callback, or
+otherwise pass the session in use to the callback. You must pass the
+C<$session> as an option to all database operations that need to be included
+in the transaction.
+
+B<Warning>: The C<$callback> can be called multiple times, so it is recommended
+to make it idempotent.
+
+A hash reference of options may be provided. these are the same as for
+L</start_transaction>.
+
 =head1 AUTHORS
 
 =over 4
