@@ -20,7 +20,7 @@ package MongoDB::Database;
 # ABSTRACT: A MongoDB Database
 
 use version;
-our $VERSION = 'v2.1.0'; # TRIAL
+our $VERSION = 'v2.1.1'; # TRIAL
 
 use MongoDB::CommandResult;
 use MongoDB::Error;
@@ -222,9 +222,10 @@ sub list_collections {
         options             => $options,
         session             => $session,
         monitoring_callback => $self->_client->monitoring_callback,
+        read_preference     => MongoDB::ReadPreference->new( mode => 'primary' ),
     );
 
-    return $self->_client->send_primary_op($op);
+    return $self->_client->send_retryable_read_op($op);
 }
 
 #pod =method collection_names
@@ -441,7 +442,7 @@ sub run_command {
         monitoring_callback => $self->_client->monitoring_callback,
     );
 
-    my $obj = $self->_client->send_read_op($op);
+    my $obj = $self->_client->send_retryable_read_op($op);
 
     return $obj->output;
 }
@@ -490,14 +491,14 @@ sub aggregate {
         $options->{maxTimeMS} = $self->max_time_ms;
     }
 
-    # read preferences are ignored if the last stage is $out
+    # read preferences are ignored if the last stage is $out or $merge
     my ($last_op) = keys %{ $pipeline->[-1] };
 
     my $op = MongoDB::Op::_Aggregate->_new(
         pipeline        => $pipeline,
         options         => $options,
         read_concern    => $self->read_concern,
-        has_out         => $last_op eq '$out',
+        has_out         => !!($last_op =~ m/\$out|\$merge/),
         client          => $self->_client,
         bson_codec      => $self->bson_codec,
         db_name         => $self->name,
@@ -509,7 +510,7 @@ sub aggregate {
         monitoring_callback => $self->_client->monitoring_callback,
     );
 
-    return $self->_client->send_read_op($op);
+    return $self->_client->send_retryable_read_op($op);
 }
 
 #pod =method watch
@@ -561,6 +562,9 @@ sub watch {
         exists($options->{resumeAfter})
             ? (resume_after => delete $options->{resumeAfter})
             : (),
+        exists($options->{startAfter})
+            ? (start_after => delete $options->{startAfter})
+            : (),
         exists($options->{maxAwaitTimeMS})
             ? (max_await_time_ms => delete $options->{maxAwaitTimeMS})
             : (),
@@ -595,7 +599,7 @@ MongoDB::Database - A MongoDB Database
 
 =head1 VERSION
 
-version v2.1.0
+version v2.1.1
 
 =head1 SYNOPSIS
 
