@@ -19,7 +19,7 @@ package MongoDB::MongoClient;
 # ABSTRACT: A connection to a MongoDB server or multi-server deployment
 
 use version;
-our $VERSION = 'v2.1.0'; # TRIAL
+our $VERSION = 'v2.1.1'; # TRIAL
 
 use Moo;
 use MongoDB::ClientSession;
@@ -36,7 +36,7 @@ use MongoDB::_Dispatcher;
 use MongoDB::_SessionPool;
 use MongoDB::_Topology;
 use MongoDB::_URI;
-use BSON 1.010001;
+use BSON 1.012000;
 use Digest::MD5;
 use UUID::URandom;
 use Tie::IxHash;
@@ -88,6 +88,9 @@ use namespace::clean -except => 'meta';
 #pod C<hostname> or C<hostname:port>), or else a L<connection string URI|/CONNECTION
 #pod STRING URI> with a seed list of one or more servers plus connection options.
 #pod
+#pod B<NOTE>: Options specified in the connection string take precedence over options
+#pod provided as constructor arguments.
+#pod
 #pod Defaults to the connection string URI C<mongodb://localhost:27017>.
 #pod
 #pod For IPv6 support, you must have a recent version of L<IO::Socket::IP>
@@ -151,9 +154,9 @@ sub _build_app_name {
 #pod * PLAIN
 #pod * SCRAM-SHA-1
 #pod
-#pod If not specified, then if no username is provided, it defaults to NONE.
-#pod If a username is provided, it is set to DEFAULT, which chooses SCRAM-SHA-1 if
-#pod available or MONGODB-CR otherwise.
+#pod If not specified, then if no username or C<authSource> URI option is provided,
+#pod it defaults to NONE.  Otherwise, it is set to DEFAULT, which chooses
+#pod SCRAM-SHA-1 if available or MONGODB-CR otherwise.
 #pod
 #pod This may be set in a connection string with the C<authMechanism> option.
 #pod
@@ -168,7 +171,8 @@ has auth_mechanism => (
 sub _build_auth_mechanism {
     my ($self) = @_;
 
-    my $default = $self->username ? 'DEFAULT' : 'NONE';
+    my $source = $self->_uri->options->{authsource} // "";
+    my $default = length( $self->username ) || length($source) ? 'DEFAULT' : 'NONE';
 
     return $self->__uri_or_else(
         u => 'authmechanism',
@@ -229,8 +233,8 @@ sub _build_bson_codec {
 
 #pod =attr compressors
 #pod
-#pod An array reference of compression type names. Currently only C<zlib>
-#pod is supported.
+#pod An array reference of compression type names. Currently, C<zlib>, C<zstd> and
+#pod C<snappy> are supported.
 #pod
 #pod =cut
 
@@ -674,12 +678,29 @@ sub _build_replica_set_name {
     );
 }
 
+#pod =attr retry_reads
+#pod
+#pod =cut
+
+has retry_reads => (
+    is      => 'lazy',
+    isa     => Boolish,
+    builder => '_build_retry_reads',
+);
+
+sub _build_retry_reads {
+    my ( $self ) = @_;
+    return $self->__uri_or_else(
+        u => 'retryreads',
+        e => 'retry_reads',
+        d => 1,
+    );
+}
+
 #pod =attr retry_writes
 #pod
 #pod Whether the client should use retryable writes for supported commands. The
-#pod default value is false, which means that no write commands will be retried.
-#pod
-#pod If this is set to a true value, then commands which support retryable writes
+#pod default value is true, which means that commands which support retryable writes
 #pod will be retried on certain errors, such as C<not master> and C<node is
 #pod recovering> errors.
 #pod
@@ -725,7 +746,7 @@ sub _build_retry_writes {
     return $self->__uri_or_else(
         u => 'retrywrites',
         e => 'retry_writes',
-        d => 0,
+        d => 1,
     );
 }
 
@@ -1097,14 +1118,23 @@ has _write_concern => (
 
 sub _build__write_concern {
     my ($self) = @_;
-    return MongoDB::WriteConcern->new(
+
+    return MongoDB::WriteConcern->new( $self->_write_concern_options );
+}
+
+# Seperated out for use in transaction option defaults
+sub _write_concern_options {
+    my ($self) = @_;
+
+    return (
+        wtimeout => $self->wtimeout,
         # Must check for defined as w can be 0, and defaults to undef
-        ( defined $self->w ? ( w        => $self->w )        : () ),
-        ( wtimeout => $self->wtimeout ),
-        ( defined $self->j        ? ( j        => $self->j )        : () ),
+        ( defined $self->w ? ( w => $self->w ) : () ),
+        ( defined $self->j ? ( j => $self->j ) : () ),
     );
 }
 
+
 #pod =method read_concern
 #pod
 #pod Returns a L<MongoDB::ReadConcern> object constructed from
@@ -1226,13 +1256,16 @@ has _credential => (
 sub _build__credential {
     my ($self) = @_;
     my $mechanism = $self->auth_mechanism;
+    my $uri_options = $self->_uri->options;
+    my $source = $uri_options->{authsource};
     my $cred = MongoDB::_Credential->new(
         monitoring_callback  => $self->monitoring_callback,
         mechanism            => $mechanism,
         mechanism_properties => $self->auth_mechanism_properties,
         ( $self->username ? ( username => $self->username ) : () ),
         ( $self->password ? ( password => $self->password ) : () ),
-        ( $self->db_name  ? ( source   => $self->db_name )  : () ),
+        ( $source ? ( source   => $source )  : () ),
+        ( $self->db_name ? ( db_name => $self->db_name ) : () ),
     );
     return $cred;
 }
@@ -1266,6 +1299,7 @@ has _dispatcher => (
         qw(
           send_direct_op
           send_primary_op
+          send_retryable_read_op
           send_read_op
           send_retryable_write_op
           send_write_op
@@ -1278,6 +1312,7 @@ sub _build__dispatcher {
     return MongoDB::_Dispatcher->new(
         topology     => $self->_topology,
         retry_writes => $self->retry_writes,
+        retry_reads  => $self->retry_reads,
     );
 }
 
@@ -1317,6 +1352,7 @@ my @deferred_options = qw(
   read_pref_tag_sets
   replica_set_name
   retry_writes
+  retry_reads
   server_selection_timeout_ms
   server_selection_try_once
   socket_check_interval_ms
@@ -1530,12 +1566,12 @@ sub start_session {
 }
 
 sub _maybe_get_implicit_session {
-    my ( $self, $opts ) = @_;
+    my ($self) = @_;
 
     # Dont return an error as implicit sessions need to be backwards compatible
     return undef unless $self->_topology->_supports_sessions; ## no critic
 
-    return $self->_start_client_session( 0, $opts );
+    return $self->_start_client_session(0);
 }
 
 sub _start_client_session {
@@ -1581,7 +1617,28 @@ sub send_admin_command {
         monitoring_callback => $self->monitoring_callback,
     );
 
-    return $self->send_read_op( $op );
+    return $self->send_retryable_read_op( $op );
+}
+
+# Ostensibly the same as above, but allows for specific addressing - uses 'send_direct_op'.
+sub _send_direct_admin_command {
+     my ( $self, $address, $command, $read_pref ) = @_;
+
+    $read_pref = MongoDB::ReadPreference->new(
+        ref($read_pref) ? $read_pref : ( mode => $read_pref ) )
+      if $read_pref && ref($read_pref) ne 'MongoDB::ReadPreference';
+
+    my $op = MongoDB::Op::_Command->_new(
+        db_name             => 'admin',
+        query               => $command,
+        query_flags         => {},
+        bson_codec          => $self->bson_codec,
+        read_preference     => $read_pref,
+        session             => $self->_maybe_get_implicit_session,
+        monitoring_callback => $self->monitoring_callback,
+    );
+
+    return $self->send_direct_op( $op, $address );
 }
 
 #--------------------------------------------------------------------------#
@@ -1609,25 +1666,20 @@ sub send_admin_command {
 
 sub list_databases {
     my ( $self, $args ) = @_;
-
     my @databases;
-    my $max_tries = 3;
-    for my $try ( 1 .. $max_tries ) {
-        last if eval {
-            my $output = $self->send_admin_command([ listDatabases => 1, ( $args ? %$args : () ) ])->output;
-            if (ref($output) eq 'HASH' && exists $output->{databases}) {
-                @databases = @{ $output->{databases} };
-            }
-            return 1;
-        } or do {
-            my $error = $@ || "Unknown error";
-            if ( $error->$_isa("MongoDB::DatabaseError" ) ) {
-                return if $error->result->output->{code} == CANT_OPEN_DB_IN_READ_LOCK() || $try < $max_tries;
-            }
-            die $error;
-        };
-    }
-
+    eval {
+        my $output = $self->send_admin_command([ listDatabases => 1, ( $args ? %$args : () ) ])->output;
+        if (ref($output) eq 'HASH' && exists $output->{databases}) {
+            @databases = @{ $output->{databases} };
+        }
+        return 1;
+    } or do {
+        my $error = $@ || "Unknown error";
+        if ( $error->$_isa("MongoDB::DatabaseError" ) ) {
+            return if $error->result->output->{code} == CANT_OPEN_DB_IN_READ_LOCK();
+        }
+        die $error;
+    };
     return @databases;
 }
 
@@ -1833,6 +1885,9 @@ sub watch {
         exists($options->{resumeAfter})
             ? (resume_after => delete $options->{resumeAfter})
             : (),
+        exists($options->{startAfter})
+            ? (start_after => delete $options->{startAfter})
+            : (),
         exists($options->{maxAwaitTimeMS})
             ? (max_await_time_ms => delete $options->{maxAwaitTimeMS})
             : (),
@@ -1876,7 +1931,7 @@ MongoDB::MongoClient - A connection to a MongoDB server or multi-server deployme
 
 =head1 VERSION
 
-version v2.1.0
+version v2.1.1
 
 =head1 SYNOPSIS
 
@@ -1945,6 +2000,9 @@ The C<host> attribute specifies either a single server to connect to (as
 C<hostname> or C<hostname:port>), or else a L<connection string URI|/CONNECTION
 STRING URI> with a seed list of one or more servers plus connection options.
 
+B<NOTE>: Options specified in the connection string take precedence over options
+provided as constructor arguments.
+
 Defaults to the connection string URI C<mongodb://localhost:27017>.
 
 For IPv6 support, you must have a recent version of L<IO::Socket::IP>
@@ -2002,9 +2060,9 @@ SCRAM-SHA-1
 
 =back
 
-If not specified, then if no username is provided, it defaults to NONE.
-If a username is provided, it is set to DEFAULT, which chooses SCRAM-SHA-1 if
-available or MONGODB-CR otherwise.
+If not specified, then if no username or C<authSource> URI option is provided,
+it defaults to NONE.  Otherwise, it is set to DEFAULT, which chooses
+SCRAM-SHA-1 if available or MONGODB-CR otherwise.
 
 This may be set in a connection string with the C<authMechanism> option.
 
@@ -2028,8 +2086,8 @@ If not provided, a L<BSON> object with default values will be generated.
 
 =head2 compressors
 
-An array reference of compression type names. Currently only C<zlib>
-is supported.
+An array reference of compression type names. Currently, C<zlib>, C<zstd> and
+C<snappy> are supported.
 
 =head2 zlib_compression_level
 
@@ -2240,12 +2298,12 @@ names must match this or they will be removed from the topology.
 
 This may be set in a connection string with the C<replicaSet> option.
 
+=head2 retry_reads
+
 =head2 retry_writes
 
 Whether the client should use retryable writes for supported commands. The
-default value is false, which means that no write commands will be retried.
-
-If this is set to a true value, then commands which support retryable writes
+default value is true, which means that commands which support retryable writes
 will be retried on certain errors, such as C<not master> and C<node is
 recovering> errors.
 
@@ -2873,6 +2931,10 @@ C<replicaSet>
 
 =item *
 
+C<retryReads>
+
+=item *
+
 C<retryWrites>
 
 =item *
@@ -2909,6 +2971,9 @@ C<zlibCompressionLevel>
 
 =back
 
+B<NOTE>: Options specified in the connection string take precedence over options
+provided as constructor arguments.
+
 See the official MongoDB documentation on connection strings for more on the URI
 format and connection string options:
 L<http://docs.mongodb.org/manual/reference/connection-string/>.
@@ -3175,6 +3240,10 @@ C<auth_mechanism_properties> attribute or in the connection string.
 You B<MUST> call the L</reconnect> method on any MongoDB::MongoClient objects
 after forking or spawning a thread.
 
+B<NOTE>: Per L<threads> documentation, use of Perl threads is discouraged by the
+maintainers of Perl and the MongoDB Perl driver does not test or provide support
+for use with threads.
+
 =head1 AUTHORS
 
 =over 4
