@@ -20,7 +20,7 @@ package MongoDB::Op::_ChangeStream;
 # and operationTime if supported
 
 use version;
-our $VERSION = 'v2.1.0'; # TRIAL
+our $VERSION = 'v2.1.1'; # TRIAL
 
 use Moo;
 
@@ -76,6 +76,11 @@ has resume_after => (
     predicate => 'has_resume_after',
 );
 
+has start_after => (
+    is => 'ro',
+    predicate => 'has_start_after',
+);
+
 has all_changes_for_cluster => (
     is => 'ro',
     isa => Boolish,
@@ -170,6 +175,10 @@ sub execute {
                 ? (resumeAfter => $self->resume_after)
                 : ()
             ),
+            ($self->has_start_after
+                ? (startAfter => $self->start_after)
+                : ()
+            ),
         }},
         @{ $self->pipeline },
     );
@@ -202,6 +211,7 @@ sub execute {
             ns         => '',
             id         => 0,
             firstBatch => ( delete $res->output->{result} ) || [],
+            postBatchResumeToken => 0,
         };
     }
 
