@@ -19,7 +19,7 @@ package MongoDB::QueryResult;
 # ABSTRACT: An iterator for Mongo query results
 
 use version;
-our $VERSION = 'v2.1.0'; # TRIAL
+our $VERSION = 'v2.1.1'; # TRIAL
 
 use Moo;
 use MongoDB::Error;
@@ -117,6 +117,13 @@ has _cursor_id => (
     isa => Any,
 );
 
+has _post_batch_resume_token => (
+    is       => 'ro',
+    required => 0,
+    writer   => '_set_post_batch_resume_token',
+    isa => Any,
+);
+
 has _cursor_start => (
     is       => 'ro',
     required => 1,
@@ -151,7 +158,14 @@ sub _add_docs {
     my $self = shift;
     push @{$self->{_docs}}, @_;
 }
-sub _next_doc { shift @{$_[0]{_docs}} }
+sub _next_doc {
+    my $self = shift;
+    my $doc = shift @{$self->{_docs}};
+    if (my $resume_token = $self->_post_batch_resume_token) {
+        $doc->{postBatchResumeToken} = $resume_token;
+    }
+    return $doc;
+}
 sub _drain_docs {
     my @docs = @{$_[0]{_docs}};
     $_[0]{_cursor_at} += scalar @docs;
@@ -259,6 +273,7 @@ sub _get_more {
     $self->_set_cursor_start( $result->{starting_from} );
     $self->_inc_cursor_num( $result->{number_returned} );
     $self->_add_docs( @{ $result->{docs} } );
+    $self->_set_post_batch_resume_token($result->{cursor}{postBatchResumeToken});
     return scalar @{ $result->{docs} };
 }
 
@@ -341,6 +356,10 @@ sub DEMOLISH {
 #pod
 #pod =head2 Multithreading
 #pod
+#pod B<NOTE>: Per L<threads> documentation, use of Perl threads is discouraged by the
+#pod maintainers of Perl and the MongoDB Perl driver does not test or provide support
+#pod for use with threads.
+#pod
 #pod Iterators are cloned in threads, but not reset.  Iterating from multiple
 #pod threads will give unpredictable results.  Only iterate from a single
 #pod thread.
@@ -361,7 +380,7 @@ MongoDB::QueryResult - An iterator for Mongo query results
 
 =head1 VERSION
 
-version v2.1.0
+version v2.1.1
 
 =head1 SYNOPSIS
 
@@ -402,6 +421,10 @@ request will be sent to the originating server to free server resources.
 
 =head2 Multithreading
 
+B<NOTE>: Per L<threads> documentation, use of Perl threads is discouraged by the
+maintainers of Perl and the MongoDB Perl driver does not test or provide support
+for use with threads.
+
 Iterators are cloned in threads, but not reset.  Iterating from multiple
 threads will give unpredictable results.  Only iterate from a single
 thread.
