@@ -19,7 +19,7 @@ use warnings;
 package MongoDB::_Protocol;
 
 use version;
-our $VERSION = 'v2.1.0'; # TRIAL
+our $VERSION = 'v2.1.1'; # TRIAL
 
 use MongoDB::_Constants;
 use MongoDB::Error;
@@ -315,14 +315,27 @@ sub write_msg {
 #     char*     compressedMessage;  // compressed contents
 # };
 
+# Note that Zlib is in perl core (since 5.9.3) so shouldnt need lazy loading
+sub _assert_zstd {
+    MongoDB::UsageError->throw(qq/Compress::Zstd must be installed to support zstd compression\n/)
+      unless eval { require Compress::Zstd };
+}
+
+sub _assert_snappy {
+    MongoDB::UsageError->throw(qq/Compress::Snappy must be installed to support snappy compression\n/)
+      unless eval { require Compress::Snappy };
+}
+
 # decompressors indexed by ID.
 my @DECOMPRESSOR = (
     # none
     sub { shift },
-    # snappy (unsupported)
-    undef,
+    # snappy
+    sub { Compress::Snappy::decompress(shift) },
     # zlib
     sub { Compress::Zlib::uncompress(shift) },
+    # zstd
+    sub { Compress::Zstd::decompress(shift) },
 );
 
 # construct compressor by name with options
@@ -335,6 +348,13 @@ sub get_compressor {
             callback => sub { shift },
         };
     }
+    elsif ($name eq 'snappy') {
+        _assert_snappy();
+        return {
+            id => 1,
+            callback => sub { Compress::Snappy::compress(shift) },
+        };
+    }
     elsif ($name eq 'zlib') {
         my $level = $comp_opt->{zlib_compression_level};
         $level = undef
@@ -349,6 +369,13 @@ sub get_compressor {
             },
         };
     }
+    elsif ($name eq 'zstd') {
+        _assert_zstd();
+        return {
+            id => 3,
+            callback => sub { Compress::Zstd::compress(shift) },
+        };
+    }
     else {
         MongoDB::ProtocolError->throw("Unknown compressor '$name'");
     }
