@@ -17,7 +17,7 @@ use warnings;
 package MongoDB::_Topology;
 
 use version;
-our $VERSION = 'v2.1.0'; # TRIAL
+our $VERSION = 'v2.1.1'; # TRIAL
 
 use Moo;
 use BSON;
@@ -55,6 +55,10 @@ use Time::HiRes qw/time usleep/;
 
 use namespace::clean;
 
+with $_ for qw(
+  MongoDB::Role::_TopologyMonitoring
+);
+
 #--------------------------------------------------------------------------#
 # attributes
 #--------------------------------------------------------------------------#
@@ -235,6 +239,13 @@ has is_compatible => (
     isa => Boolish,
 );
 
+has compatibility_error => (
+    is      => 'ro',
+    default => '',
+    writer => '_set_compatibility_error',
+    isa     => Str,
+);
+
 has wire_version_floor => (
     is => 'ro',
     writer => '_set_wire_version_floor',
@@ -343,6 +354,12 @@ sub _build_handshake_document {
 
 sub BUILD {
     my ($self) = @_;
+
+    $self->publish_topology_opening
+      if $self->monitoring_callback;
+
+    $self->publish_old_topology_desc
+      if $self->monitoring_callback;
     my $type = $self->type;
     my @addresses = @{ $self->uri->hostids };
 
@@ -369,9 +386,51 @@ sub BUILD {
 
     $self->_add_address_as_unknown($_) for @addresses;
 
+    $self->publish_new_topology_desc
+      if $self->monitoring_callback;
+
     return;
 }
 
+sub DEMOLISH {
+    my $self = shift;
+
+    $self->publish_topology_closing
+      if $self->monitoring_callback;
+
+    return;
+}
+
+sub _check_for_uri_changes {
+    my ($self) = @_;
+
+    my $type = $self->type;
+    return unless
+        $type eq 'Sharded'
+        or $type eq 'Unknown';
+
+    my @existing = @{ $self->uri->hostids };
+
+    my $options = {
+        fallback_ttl_sec => $self->{heartbeat_frequency_sec},
+    };
+
+    if ($self->uri->check_for_changes($options)) {
+        my %new = map { ($_, 1) } @{ $self->uri->hostids };
+        for my $address (@existing) {
+            if (!$new{$address}) {
+                $self->_remove_address($address);
+            }
+            else {
+                delete $new{$address};
+            }
+        }
+        for my $address (keys %new) {
+            $self->_add_address_as_unknown($address);
+        }
+    }
+}
+
 #--------------------------------------------------------------------------#
 # public methods
 #--------------------------------------------------------------------------#
@@ -401,8 +460,24 @@ sub close_all_links {
     return;
 }
 
+sub _maybe_get_txn_error_labels_and_unpin_from {
+    my $op = shift;
+    return () unless defined $op
+        && defined $op->session;
+    if ( $op->session->_in_transaction_state( TXN_STARTING, TXN_IN_PROGRESS ) ) {
+        $op->session->_unpin_address;
+        return ( error_labels => [ TXN_TRANSIENT_ERROR_MSG ] );
+    } elsif ( $op->session->_in_transaction_state( TXN_COMMITTED ) ) {
+        return ( error_labels => [ TXN_UNKNOWN_COMMIT_MSG ] );
+    }
+    return ();
+}
+
 sub get_readable_link {
-    my ( $self, $read_pref ) = @_;
+    my ( $self, $op ) = @_;
+    $self->_check_for_uri_changes;
+
+    my $read_pref = defined $op ? $op->read_preference : undef;
 
     my $mode = $read_pref ? lc $read_pref->mode : 'primary';
     my $method =
@@ -428,25 +503,33 @@ sub get_readable_link {
     }
 
     my $rp = $read_pref ? $read_pref->as_string : 'primary';
+
     MongoDB::SelectionError->throw(
-        "No readable server available for matching read preference $rp. MongoDB server status:\n"
-          . $self->_status_string );
+        message => "No readable server available for matching read preference $rp. MongoDB server status:\n"
+          . $self->_status_string,
+        _maybe_get_txn_error_labels_and_unpin_from( $op ),
+    );
 }
 
 sub get_specific_link {
-    my ( $self, $address ) = @_;
+    my ( $self, $address, $op ) = @_;
+    $self->_check_for_uri_changes;
 
     my $server = $self->servers->{$address};
     if ( $server && ( my $link = $self->_get_server_link($server) ) ) {
         return $link;
     }
     else {
-        MongoDB::SelectionError->throw("Server $address is no longer available");
+        MongoDB::SelectionError->throw(
+            message => "Server $address is no longer available",
+            _maybe_get_txn_error_labels_and_unpin_from( $op ),
+        );
     }
 }
 
 sub get_writable_link {
-    my ($self) = @_;
+    my ( $self, $op ) = @_;
+    $self->_check_for_uri_changes;
 
     my $method =
       ( $self->type eq "Single" || $self->type eq "Sharded" )
@@ -470,12 +553,17 @@ sub get_writable_link {
     }
 
     MongoDB::SelectionError->throw(
-        "No writable server available.  MongoDB server status:\n" . $self->_status_string );
+        message => "No writable server available.  MongoDB server status:\n" . $self->_status_string,
+        _maybe_get_txn_error_labels_and_unpin_from( $op ),
+    );
 }
 
+# Marking a server unknown from outside the topology indicates an operational
+# error, so the last scan is set to EPOCH so that the next scan won't wait for
+# the scanning cooldown.
 sub mark_server_unknown {
-    my ( $self, $server, $error ) = @_;
-    $self->_reset_address_to_unknown( $server->address, $error );
+    my ( $self, $server, $error, $no_cooldown ) = @_;
+    $self->_reset_address_to_unknown( $server->address, $error, $no_cooldown // EPOCH );
     return;
 }
 
@@ -558,6 +646,9 @@ sub _add_address_as_unknown {
     $error = $error ? "$error" : "";
     $error =~ s/ at \S+ line \d+.*//ms;
 
+    $self->publish_server_opening($address)
+      if $self->monitoring_callback;
+
     return $self->servers->{$address} = MongoDB::_Server->new(
         address          => $address,
         last_update_time => $last_update || EPOCH,
@@ -655,8 +746,19 @@ sub _supports_transactions {
     my ( $self ) = @_;
 
     return unless $self->_supports_sessions;
+    return $self->_supports_mongos_pinning_transactions if $self->type eq 'Sharded';
     return if $self->wire_version_ceil < 7;
-    return if $self->type eq 'Sharded';
+    return 1;
+}
+
+# Seperated out so can be used in dispatch logic
+sub _supports_mongos_pinning_transactions {
+    my ( $self ) = @_;
+
+    # Separated out so that it doesnt return 1 for wire version 8 non sharded
+    return if $self->wire_version_ceil < 8;
+    # This extra sharded check is required so this test can be standalone
+    return if $self->type ne 'Sharded';
     return 1;
 }
 
@@ -835,7 +937,6 @@ sub _get_server_in_latency_window {
     my @sorted =
       sort { $a->{rtt} <=> $b->{rtt} }
       map { { server => $_, rtt => $rtt_hash->{ $_->address } } } @$servers;
-
     # lowest RTT is always in the windows
     my @in_window = shift @sorted;
 
@@ -944,6 +1045,8 @@ sub _remove_address {
         $self->_clear_current_primary;
     }
     delete $self->$_->{$address} for qw/servers links rtt_ewma_sec/;
+    $self->publish_server_closing( $address )
+      if $self->monitoring_callback;
     return;
 }
 
@@ -955,7 +1058,7 @@ sub _remove_server {
 
 sub _reset_address_to_unknown {
     my ( $self, $address, $error, $update_time ) = @_;
-    $update_time ||= time;
+    $update_time //= time;
 
     $self->_remove_address($address);
     my $desc = $self->_add_address_as_unknown( $address, $update_time, $error );
@@ -1036,6 +1139,7 @@ sub _selection_timeout {
                     );
                 }
             }
+            $self->_set_compatibility_error($error_string);
             MongoDB::ProtocolError->throw( $error_string );
         }
 
@@ -1080,6 +1184,9 @@ sub _generate_ismaster_request {
 sub _update_topology_from_link {
     my ( $self, $link, %opts ) = @_;
 
+    $self->publish_server_heartbeat_started( $link )
+      if $self->monitoring_callback;
+
     my $start_time = time;
     my $is_master = eval {
         my $op = MongoDB::Op::_Command->_new(
@@ -1097,6 +1204,10 @@ sub _update_topology_from_link {
         $op->execute( $link )->output;
     };
     if ( my $e = $@ ) {
+        my $end_time_fail = time;
+        my $rtt_sec_fail = $end_time_fail - $start_time;
+        $self->publish_server_heartbeat_failed( $link, $rtt_sec_fail, $e )
+          if $self->monitoring_callback;
         if ($e->$_isa("MongoDB::DatabaseError") && $e->code == USER_NOT_FOUND ) {
             MongoDB::AuthError->throw("mechanism negotiation error: $e");
         }
@@ -1124,6 +1235,11 @@ sub _update_topology_from_link {
 
     my $end_time = time;
     my $rtt_sec = $end_time - $start_time;
+    # Protect against clock skew
+    $rtt_sec = 0 if $rtt_sec < 0;
+
+    $self->publish_server_heartbeat_succeeded( $link, $rtt_sec, $is_master )
+      if $self->monitoring_callback;
 
     my $new_server = MongoDB::_Server->new(
         address          => $link->address,
@@ -1169,12 +1285,16 @@ sub _update_topology_from_server_desc {
     # after a server has been removed
     return unless $self->servers->{$address};
 
+    $self->publish_old_topology_desc( $address, $new_server )
+      if $self->monitoring_callback;
+
     $self->_update_ewma( $address, $new_server );
 
     # must come after ewma update
     $self->servers->{$address} = $new_server;
 
     my $method = "_update_" . $self->type;
+
     $self->$method( $address, $new_server );
 
     # if link is still around, tag it with server specifics
@@ -1182,6 +1302,8 @@ sub _update_topology_from_server_desc {
 
     $self->_update_ls_timeout_minutes( $new_server );
 
+    $self->publish_new_topology_desc if $self->monitoring_callback;
+
     return $new_server;
 }
 
