@@ -22,6 +22,7 @@ use MongoDB;
 use BSON;
 use Test::More;
 use Path::Tiny;
+use Scalar::Util qw/isvstring/;
 use boolean;
 use version;
 
@@ -35,6 +36,8 @@ our @EXPORT_OK = qw(
   skip_if_mongod
   skip_unless_mongod
   skip_unless_failpoints_available
+  set_failpoint
+  clear_failpoint
   skip_unless_sessions
   skip_unless_transactions
   to_snake_case
@@ -44,6 +47,7 @@ our @EXPORT_OK = qw(
   get_features
   check_min_server_version
   uuid_to_string
+  skip_unless_min_version
 );
 
 my @testdbs;
@@ -88,7 +92,7 @@ sub build_client {
         $host,
         {
             ssl                         => $ssl || $ENV{MONGO_SSL},
-            socket_timeout_ms           => 60000,
+            socket_timeout_ms           => 120000,
             server_selection_timeout_ms => $ENV{ATLAS_PROXY} ? 10000 : 2000,
             server_selection_try_once   => 0,
             wtimeout                    => 15000,
@@ -124,8 +128,10 @@ sub get_capped {
 }
 
 sub skip_unless_mongod {
+    my $min_version = shift;
+    my $conn;
     eval {
-        my $conn = build_client( server_selection_timeout_ms => 10000 );
+        $conn = build_client( server_selection_timeout_ms => 10000 );
         my $topo = $conn->_topology;
         $topo->scan_all_servers;
         my $link;
@@ -153,6 +159,11 @@ sub skip_unless_mongod {
         }
         plan skip_all => "$err";
     }
+
+    if ($min_version) {
+        plan skip_all => "Requires MongoDB $min_version"
+            if check_min_server_version($conn, $min_version);
+    }
 }
 
 sub skip_if_mongod {
@@ -202,8 +213,47 @@ sub skip_unless_failpoints_available {
     plan skip_all => "enableTestCommands is off"
       unless $param && $param->{enableTestCommands};
 
+    # For transaction tests under mongos
     plan skip_all => "fail points not supported via mongos"
-      if $server_type eq 'Mongos';
+      if $server_type eq 'Mongos'
+        && ( $conn->_topology->wire_version_ceil < 8
+          || $arg->{skip_mongos} );
+}
+
+sub set_failpoint {
+    my ( $client, $failpoint ) = @_;
+
+    return unless defined $failpoint;
+    _send_failpoint_admin_command( $client, [
+        configureFailPoint => $failpoint->{configureFailPoint},
+        mode => $failpoint->{mode},
+        defined $failpoint->{data}
+          ? ( data => $failpoint->{data} )
+          : (),
+    ]);
+}
+
+sub clear_failpoint {
+    my ( $client, $failpoint ) = @_;
+
+    return unless defined $failpoint;
+    _send_failpoint_admin_command( $client, [
+        configureFailPoint => $failpoint->{configureFailPoint},
+        mode => 'off',
+    ]);
+}
+
+# Failpoint commands must be sent to all servers if sharded
+sub _send_failpoint_admin_command {
+    my ( $client, $command ) = @_;
+
+    if ( $client->_topology->type eq 'Sharded' ) {
+        for my $server ( $client->_topology->all_servers ) {
+            $client->_send_direct_admin_command( $server->address, $command );
+        }
+    } else {
+        $client->send_admin_command( $command );
+    }
 }
 
 sub skip_unless_sessions {
@@ -227,9 +277,13 @@ sub server_version {
 
 sub check_min_server_version {
     my ( $conn, $min_version ) = @_;
-    $min_version = "v$min_version" unless $min_version =~ /^v/;
-    $min_version .= ".0" unless $min_version =~ /^v\d+\.\d+.\d+$/;
-    $min_version = version->new($min_version);
+    unless ( ref($min_version) eq 'version') {
+        unless ( isvstring($min_version) ) {
+            $min_version = "v$min_version" unless $min_version =~ /^v/;
+            $min_version .= ".0" unless $min_version =~ /^v\d+\.\d+.\d+$/;
+        }
+        $min_version = version->new($min_version);
+    }
     my $server_version = server_version($conn);
     if ( $min_version > $server_version ) {
         return 1;
@@ -237,6 +291,12 @@ sub check_min_server_version {
     return 0;
 }
 
+sub skip_unless_min_version {
+    my ( $conn, $min_version ) = @_;
+    plan skip_all => "Requires MongoDB $min_version"
+        if check_min_server_version( $conn, $min_version );
+}
+
 sub server_type {
 
     my $conn = shift;
